         PRESERVE8 ; Indicate the code here preserve 
                   ; 8 byte stack alignment
         THUMB     ; Indicate THUMB code is used
         AREA    |.text|, CODE, READONLY   ; Start of CODE area

         INCLUDE LPC11xx.hs
         ; Comment out the following line for LPCXpresso
         GBLA    KEIL_MCB1000_BOARD

         EXPORT Blinky
Blinky   FUNCTION
         ; Switch Clock to 48MHz
         BL Set48MHzClock

         IF :DEF:KEIL_MCB1000_BOARD
         ; Initialize LED (GPIO #2, bit[0]) output
         ELSE
         ; Initialize LED (GPIO #0, bit[7]) output
         ENDIF
         BL LedOutputCfg

         ; Program SysTick timer at 1KHz.
         ; At 48MHz, SysTick trigger every 48000 CPU cycles
         ;  SysTick->LOAD = (48000-1); // Count from 47999 to 0
         ;  SysTick->VAL  = 0;   // Clear SysTick value
         ;  SysTick->CTRL = 0x5; // Enable, using core clock
         LDR     R0,=SysTick_BASE
         LDR     R1,=47999
         STR     R1,[R0,#SysTick_LOAD]
         MOVS    R1, #0
         STR     R1,[R0,#SysTick_VAL]
         MOVS    R1, #5
         STR     R1,[R0,#SysTick_CTRL]

         ; while(1){	 // Blink at 1Hz
Blinky_loop
         ; for (i=0;i<500;i++) { // Wait for 0.5 seconds
	 ;     while ((SysTick->CTRL & 0x10000)==0); // Wait for counter underflow
	 ; }
         LDR     R1, =500
Blinky_inner_loop1
Blinky_inner_loop2
         LDR     R2, =0x10000
         LDR     R3, [R0,#SysTick_CTRL]
         TST     R3, R2
         BEQ     Blinky_inner_loop2
         SUBS    R1, R1, #1
         BNE     Blinky_inner_loop1

         ; Toggle bit 7
         ; LPC_GPIO0->MASKED_ACCESS[1<<7] = ~LPC_GPIO0->MASKED_ACCESS[1<<7];

         IF :DEF:KEIL_MCB1000_BOARD
         LDR     R1,=LPC_GPIO2_BASE
         MOVS    R2,#0x4 ; (0x01 * 4)
         LDR     R3, [R1, R2]
         MVNS    R3, R3
         STR     R3, [R1, R2]        
         ELSE
         LDR     R1,=LPC_GPIO0_BASE
         LDR     R2,=0x200 ; (0x80 * 4)
         LDR     R3, [R1, R2]
         MVNS    R3, R3
         STR     R3, [R1, R2]        
         ENDIF

         ;  } // end while
         B       Blinky_loop
         ENDFUNC
; --------------------------------------------------
Set48MHzClock FUNCTION
         LDR     R0,=LPC_SYSCON_BASE

         ; Power up the PLL and System oscillator	
         ; (clear the powerdown bits for PLL and System oscillator)
         ; LPC_SYSCON->PDRUNCFG = LPC_SYSCON->PDRUNCFG & 0xFFFFFF5F;
         LDR     R3, =PDRUNCFG
         LDR     R1,[R0, R3]
         MOVS    R2, #0xA0
         BICS    R1, R1, R2
         STR     R1,[R0, R3]

         ; Select PLL source as crystal oscillator
         ;  0 - IRC oscillator
         ;  1 - System oscillator
         ;  2 - WDT oscillator
         ; LPC_SYSCON->SYSPLLCLKSEL = 1;
         MOVS    R1, #0x1
         STR     R1,[R0, #SYSPLLCLKSEL]
 

         ; Update SYSPLL setting (0->1 sequence)
         ; LPC_SYSCON->SYSPLLCLKUEN = 0;
         MOVS    R1, #0x0
         STR     R1,[R0, #SYSPLLCLKUEN]

         ; LPC_SYSCON->SYSPLLCLKUEN = 1;
         MOVS    R1, #0x1
         STR     R1,[R0, #SYSPLLCLKUEN]

         ; Set PLL to 48MHz generate from 12MHz
         ;  M = 48/12 = 4 (MSEL = 3)
         ;  FCCO (must be between 156 to 320MHz, and is 2x, 4x, 8x or 16x of Clock) 
         ;  Clock freq out selected as 192MHz
         ;  P = 192MHz/48MHz/2 = 2 (PSEL = 1)
         ;   bit[8]   - BYPASS
         ;   bit[7]   - DIRECT
         ;   bit[6:5] - PSEL  (1,2,4,8)
         ;   bit[4:0] - MSEL  (1-32)
         ; LPC_SYSCON->SYSPLLCTRL = (3 + (1<<5)); // M = 4, P = 2
         MOVS    R1, #0x23
         STR     R1,[R0, #SYSPLLCTRL]

         ; wait until PLL is locked
         ; while(LPC_SYSCON->SYSPLLSTAT == 0);
Set48MHzClock_waitloop1
         LDR     R1,[R0,#SYSPLLSTAT]
         CMP     R1, #0
         BEQ     Set48MHzClock_waitloop1

         ; Switch main clock to PLL clock
         ;    0 - IRC
         ;    1 - Input clock to system PLL
         ;    2 - WDT clock
         ;    3 - System PLL output
         ; LPC_SYSCON->MAINCLKSEL = 3;
         MOVS    R1, #0x3
         STR     R1,[R0, #MAINCLKSEL]

         ; Update Main Clock Select setting (0->1 sequence)
         ; LPC_SYSCON->MAINCLKUEN = 0;
         MOVS    R1, #0x0
         STR     R1,[R0, #MAINCLKUEN]

         ; LPC_SYSCON->MAINCLKUEN = 1;
         MOVS    R1, #0x1
         STR     R1,[R0, #MAINCLKUEN]
         BX      LR
         ENDFUNC
; --------------------------------------------------
LedOutputCfg FUNCTION

         ; Enable clock to IO configuration block (bit[16] of AHBCLOCK Control register)
         ; and enable clock to GPIO (bit[6] of AHBCLOCK Control register
         ; LPC_SYSCON->SYSAHBCLKCTRL = LPC_SYSCON->SYSAHBCLKCTRL | (1<<16) | (1<<6);
         LDR     R0,=(LPC_SYSCON_BASE+SYSAHBCLKCTRL)
         LDR     R2,=0x10040  ; (1<<16) | (1<<6)
         LDR     R1,[R0]
         ORRS    R1, R1, R2
         STR     R1,[R0]

         IF :DEF:KEIL_MCB1000_BOARD
         ; PIO2_0 IO output config
         ;  bit[5]   - Hysteresis (0=disable, 1 =enable)
         ;  bit[4:3] - MODE(0=inactive, 1 =pulldown, 2=pullup, 3=repeater)
         ;  bit[2:0] - Function (0 = IO, 1=DTR, 2=SSEL1)
         ; LPC_IOCON->PIO2_0 = (0x0) + (0<<3) + (0<<5);
         LDR     R0,=LPC_IOCON_BASE
         MOVS    R1, #0x0
         STR     R1, [R0,#PIO2_0]

         ; Initial bit[0] output is 0
         ; LPC_GPIO2->MASKED_ACCESS[1<<0] = 0;
         LDR     R0,=LPC_GPIO2_BASE
         MOVS    R1, #0
         MOVS    R2, #0x1   ; (1<<0)
         LSLS    R2, R2, #2 ; (R2 = (1<<0) x 4 (bytes)
         STR     R1, [R0, R2]         

         ; Set pin 0 as output
         ; LPC_GPIO2->DIR = LPC_GPIO2->DIR | (1<<0);
         LDR     R0,=LPC_GPIO2_REGBASE
         MOVS    R1, #0x01
         STR     R1, [R0,#GPIO_DIR]

         ELSE
         ; PIO0_7 IO output config
         ;  bit[5]   - Hysteresis (0=disable, 1 =enable)
         ;  bit[4:3] - MODE(0=inactive, 1 =pulldown, 2=pullup, 3=repeater)
         ;  bit[2:0] - Function (0 = IO, 1=CTS)
         ; LPC_IOCON->PIO0_7 = (0x0) + (0<<3) + (0<<5);
         LDR     R0,=LPC_IOCON_BASE
         MOVS    R1, #0x0
         STR     R1, [R0,#PIO0_7]

         ; Initial bit[7] output is 0
         ; LPC_GPIO0->MASKED_ACCESS[1<<7] = 0;
         LDR     R0,=LPC_GPIO0_BASE
         MOVS    R1, #0
         MOVS    R2, #0x80  ; (1<<7)
         LSLS    R2, R2, #2 ; (R2 = (1<<7) x 4 (bytes)
         STR     R1, [R0, R2]         

         ; Set pin 7 as output
         ; LPC_GPIO0->DIR = LPC_GPIO0->DIR | (1<<7);
         LDR     R0,=LPC_GPIO0_REGBASE
         MOVS    R1, #0x80
         STR     R1, [R0,#GPIO_DIR]
         ENDIF
         BX      LR
         ENDFUNC
; --------------------------------------------------
         ALIGN
         END
; --------------------------------------------------
