         PRESERVE8 ; Indicate the code here preserve 
                   ; 8 byte stack alignment
         THUMB     ; Indicate THUMB code is used
; --------------------------------------------------
; Allocate data variable space
         AREA    | Header Data|, DATA   ; Start of Data definitions
         ALIGN   4

MyData1  DCD     0    ; Word size data
MyData2  DCW     0    ; half Word size data
MyData3  DCB     0    ; byte size data
; --------------------------------------------------
         AREA    |.text|, CODE, READONLY   ; Start of CODE area
         INCLUDE LPC11xx.hs
         EXPORT DataTest
DataTest FUNCTION
         ; Switch Clock to 48MHz
         BL Set48MHzClock

         LDR     R0,=MyData1
         LDR     R1,=0x00001234
         STR     R1,[R0]  ; MyData1 = 0x00001234

         LDR     R0,=MyData2
         LDR     R1,=0x55CC
         STRH    R1,[R0]  ; MyData2 = 0x55CC

         LDR     R0,=MyData3
         LDR     R1,=0xAA
         STRB    R1,[R0]  ; MyData3 = 0xAA

         B       . ; Endless loop
         ENDFUNC
; --------------------------------------------------
Set48MHzClock FUNCTION
         LDR     R0,=LPC_SYSCON_BASE

         ; Power up the PLL and System oscillator	
         ; (clear the powerdown bits for PLL and System oscillator)
         ; LPC_SYSCON->PDRUNCFG = LPC_SYSCON->PDRUNCFG & 0xFFFFFF5F;
         LDR     R3, =PDRUNCFG
         LDR     R1,[R0, R3]
         MOVS    R2, #0xA0
         BICS    R1, R1, R2
         STR     R1,[R0, R3]

         ; Select PLL source as crystal oscillator
         ;  0 - IRC oscillator
         ;  1 - System oscillator
         ;  2 - WDT oscillator
         ; LPC_SYSCON->SYSPLLCLKSEL = 1;
         MOVS    R1, #0x1
         STR     R1,[R0, #SYSPLLCLKSEL]
 

         ; Update SYSPLL setting (0->1 sequence)
         ; LPC_SYSCON->SYSPLLCLKUEN = 0;
         MOVS    R1, #0x0
         STR     R1,[R0, #SYSPLLCLKUEN]

         ; LPC_SYSCON->SYSPLLCLKUEN = 1;
         MOVS    R1, #0x1
         STR     R1,[R0, #SYSPLLCLKUEN]

         ; Set PLL to 48MHz generate from 12MHz
         ;  M = 48/12 = 4 (MSEL = 3)
         ;  FCCO (must be between 156 to 320MHz, and is 2x, 4x, 8x or 16x of Clock) 
         ;  Clock freq out selected as 192MHz
         ;  P = 192MHz/48MHz/2 = 2 (PSEL = 1)
         ;   bit[8]   - BYPASS
         ;   bit[7]   - DIRECT
         ;   bit[6:5] - PSEL  (1,2,4,8)
         ;   bit[4:0] - MSEL  (1-32)
         ; LPC_SYSCON->SYSPLLCTRL = (3 + (1<<5)); // M = 4, P = 2
         MOVS    R1, #0x23
         STR     R1,[R0, #SYSPLLCTRL]

         ; wait until PLL is locked
         ; while(LPC_SYSCON->SYSPLLSTAT == 0);
Set48MHzClock_waitloop1
         LDR     R1,[R0,#SYSPLLSTAT]
         CMP     R1, #0
         BEQ     Set48MHzClock_waitloop1

         ; Switch main clock to PLL clock
         ;    0 - IRC
         ;    1 - Input clock to system PLL
         ;    2 - WDT clock
         ;    3 - System PLL output
         ; LPC_SYSCON->MAINCLKSEL = 3;
         MOVS    R1, #0x3
         STR     R1,[R0, #MAINCLKSEL]

         ; Update Main Clock Select setting (0->1 sequence)
         ; LPC_SYSCON->MAINCLKUEN = 0;
         MOVS    R1, #0x0
         STR     R1,[R0, #MAINCLKUEN]

         ; LPC_SYSCON->MAINCLKUEN = 1;
         MOVS    R1, #0x1
         STR     R1,[R0, #MAINCLKUEN]
         BX      LR
         ENDFUNC
; --------------------------------------------------
         ALIGN
         END
; --------------------------------------------------
