         PRESERVE8 ; Indicate the code here preserve 
                   ; 8 byte stack alignment
         THUMB     ; Indicate THUMB code is used
         AREA    |.text|, CODE, READONLY   ; Start of CODE area

         INCLUDE LPC11xx.hs
         EXPORT UartTest
UartTest   FUNCTION
         BL Set48MHzClock      ; Switch Clock to 48MHz
         BL UartConfig         ; Initialize UART

         ; UartPuts ("Hello\n");
         LDR     R0,=Hello_message ; start address of hello message
         BL      UartPuts      ; print message
UartTest_loop
         BL      UartGetRxDataAvail ; check if data are received
         CMP     R0, #0
         BEQ     UartTest_loop ; Return value =0 (no data), try again
         BL      UartGetRxData ; Get receive data in R0
         BL      UartPutc      ; output receive data
         B       UartTest_loop
Hello_message
         DCB     "Hello\n", 0
         ALIGN
         ENDFUNC
; --------------------------------------------------
UartGetRxDataAvail FUNCTION
         ; Return 1 if data received
         LDR     R0,=LPC_UART_BASE
         LDR     R0, [R0, #UART_LSR]
         MOVS    R1, #1
         ANDS    R0, R0, R1
         BX      LR
         ENDFUNC
; --------------------------------------------------
UartGetRxData FUNCTION
         ; Return received data in R0
         LDR     R0,=LPC_UART_BASE
         LDR     R0, [R0, #UART_RBR]
         BX      LR
         ENDFUNC
; --------------------------------------------------
UartPutc FUNCTION
         ; Print a character to UART
         ; Input R0 - value of character to be printed
         LDR     R1,=LPC_UART_BASE
         MOVS    R3, #0x20  ; LSR.transmit holding register empty
         CMP     R0,#10 ; new line character
         BNE     UartPutc_wait_2

         ; while ((LPC_UART->LSR & (1<<5))==0); 
UartPutc_wait_1
         LDR     R2, [R1,#UART_LSR]
         TST     R2, R3 ; Check transmit empty
         BEQ     UartPutc_wait_1
         ; LPC_UART->THR = 13 (for correct display on Hyperterminal)
         MOVS    R2, #13 ; carriage return
         STR     R2, [R1, #UART_THR]

UartPutc_wait_2
         LDR     R2, [R1,#UART_LSR]
         TST     R2, R3 ; Check transmit empty
         BEQ     UartPutc_wait_2
         ; LPC_UART->THR = data
         STR     R0, [R1, #UART_THR]

         CMP     R0,#13 ; new line character
         BNE     UartPutc_end
         ; while ((LPC_UART->LSR & (1<<5))==0); 
UartPutc_wait_3
         LDR     R2, [R1,#UART_LSR]
         TST     R2, R3 ; Check transmit empty
         BEQ     UartPutc_wait_3
         ; LPC_UART->THR = 10 (for correct display on Hyperterminal)
         MOVS    R2, #10 ; new line
         STR     R2, [R1, #UART_THR]

UartPutc_end
         BX      LR
         ENDFUNC
; --------------------------------------------------
UartPuts FUNCTION
         ; Print a text string to UART
         ; Input R0 - starting address of string
         PUSH    {R4, LR}
         MOV     R4, R0
UartPuts_loop
         LDRB    R0, [R4] ; Read one character
         CMP     R0, #0
         BEQ     UartPuts_end
         BL      UartPutc
         ADDS    R4, R4, #1
         B       UartPuts_loop
UartPuts_end
         POP     {R4, PC}
         ENDFUNC
; --------------------------------------------------
UartConfig FUNCTION
         ; UART interface are : PIO1_7 (TXD) and PIC1_6 (RXD)
         ; Other UART signals (DTR, DSR, CTS, RTS, RI) are not used

         ; Enable clock to IO configuration block 
         ; (bit[16] of AHBCLOCK Control register)
         ; LPC_SYSCON->SYSAHBCLKCTRL = LPC_SYSCON->SYSAHBCLKCTRL | (1<<16);
         LDR     R0,=(LPC_SYSCON_BASE+SYSAHBCLKCTRL)
         LDR     R2,=0x10000  ; (1<<16)
         LDR     R1,[R0]
         ORRS    R1, R1, R2
         STR     R1,[R0]

         ; PIO1_7 IO output config
         ;  bit[5]   - Hysteresis (0=disable, 1 =enable)
         ;  bit[4:3] - MODE(0=inactive, 1 =pulldown, 2=pullup, 3=repeater)
         ;  bit[2:0] - Function (0 = IO, 1=TXD, 2=CT32B0_MAT1)
         ; LPC_IOCON->PIO1_7 = (0x1) + (0<<3) + (0<<5);
         LDR     R0,=LPC_IOCON_BASE
         MOVS    R1,#PIO1_7
         MOVS    R2,#0x01
         STR     R2,[R0, R1]

         ;  PIO1_6 IO output config
         ;  bit[5]   - Hysteresis (0=disable, 1 =enable)
         ;  bit[4:3] - MODE(0=inactive, 1 =pulldown, 2=pullup, 3=repeater)
         ;  bit[2:0] - Function (0 = IO, 1=RXD, 2=CT32B0_MAT0)
         ; LPC_IOCON->PIO1_6 = (0x1) + (2<<3) + (1<<5);
         MOVS    R1,#PIO1_6
         MOVS    R2,#0x31
         STR     R2,[R0, R1]

         ; Enable clock to UART (bit[12] of AHBCLOCK Control register
         ;  LPC_SYSCON->SYSAHBCLKCTRL = LPC_SYSCON->SYSAHBCLKCTRL | (1<<12);
         LDR     R0,=(LPC_SYSCON_BASE+SYSAHBCLKCTRL)
         LDR     R2,=0x1000  ; (1<<12)
         LDR     R1,[R0]
         ORRS    R1, R1, R2
         STR     R1,[R0]

         ; UART_PCLK divide ratio = 1
         ;  LPC_SYSCON->UARTCLKDIV = 1;
         LDR     R0,=(LPC_SYSCON_BASE+UARTCLKDIV)
         MOVS    R1,#1
         STR     R1,[R0]

         ; UART_PCLK = 48MHz, Baudrate = 38400, divide ratio = 1250
         ; Line Control Register
         ; LPC_UART->LCR = (1<<7) |   // Enable access to Divisor Latches
         ;   (0<<6) |   // Disable Break Control
         ;   (0<<4) |   // Bit[5:4] parity select (odd, even, sticky-1, sticky-0)
         ;   (0<<3) |   // parity disabled
         ;   (0<<2) |   // 1 stop bit
         ;   (3<<0);    // 8-bit data
         LDR     R0,=LPC_UART_BASE
         MOVS    R1,#0x83
         STR     R1,[R0,#UART_LCR]

         ; LPC_UART->DLL = 78;  // Divisor Latch Least Significant Byte 
         ;                      // 48MHz/38400/16 = 78.125
         MOVS    R1,#78
         STR     R1,[R0,#UART_DLL]

         ; LPC_UART->DLM = 0;   // Divisor Latch Most Significant Byte  : 0
         MOVS    R1,#0
         STR     R1,[R0,#UART_DLM]

         ; LPC_UART->LCR = (0<<7) |   // Disable access to Divisor Latches
         ;   (0<<6) |   // Disable Break Control
         ;   (0<<4) |   // Bit[5:4] parity select (odd, even, sticky-1, sticky-0)
         ;   (0<<3) |   // parity disabled
         ;   (0<<2) |   // 1 stop bit
         ;   (3<<0);    // 8-bit data
         MOVS    R1,#0x3
         STR     R1,[R0,#UART_LCR]

         ; LPC_UART->FCR = 1; // Enable FIFO
         MOVS    R1,#1
         STR     R1,[R0,#UART_FCR]

         BX      LR
         ENDFUNC
; --------------------------------------------------
Set48MHzClock FUNCTION
         LDR     R0,=LPC_SYSCON_BASE

         ; Power up the PLL and System oscillator	
         ; (clear the powerdown bits for PLL and System oscillator)
         ; LPC_SYSCON->PDRUNCFG = LPC_SYSCON->PDRUNCFG & 0xFFFFFF5F;
         LDR     R3, =PDRUNCFG
         LDR     R1,[R0, R3]
         MOVS    R2, #0xA0
         BICS    R1, R1, R2
         STR     R1,[R0, R3]

         ; Select PLL source as crystal oscillator
         ;  0 - IRC oscillator
         ;  1 - System oscillator
         ;  2 - WDT oscillator
         ; LPC_SYSCON->SYSPLLCLKSEL = 1;
         MOVS    R1, #0x1
         STR     R1,[R0, #SYSPLLCLKSEL]
 

         ; Update SYSPLL setting (0->1 sequence)
         ; LPC_SYSCON->SYSPLLCLKUEN = 0;
         MOVS    R1, #0x0
         STR     R1,[R0, #SYSPLLCLKUEN]

         ; LPC_SYSCON->SYSPLLCLKUEN = 1;
         MOVS    R1, #0x1
         STR     R1,[R0, #SYSPLLCLKUEN]

         ; Set PLL to 48MHz generate from 12MHz
         ;  M = 48/12 = 4 (MSEL = 3)
         ;  FCCO (must be between 156 to 320MHz, and is 2x, 4x, 8x or 16x of Clock) 
         ;  Clock freq out selected as 192MHz
         ;  P = 192MHz/48MHz/2 = 2 (PSEL = 1)
         ;   bit[8]   - BYPASS
         ;   bit[7]   - DIRECT
         ;   bit[6:5] - PSEL  (1,2,4,8)
         ;   bit[4:0] - MSEL  (1-32)
         ; LPC_SYSCON->SYSPLLCTRL = (3 + (1<<5)); // M = 4, P = 2
         MOVS    R1, #0x23
         STR     R1,[R0, #SYSPLLCTRL]

         ; wait until PLL is locked
         ; while(LPC_SYSCON->SYSPLLSTAT == 0);
Set48MHzClock_waitloop1
         LDR     R1,[R0,#SYSPLLSTAT]
         CMP     R1, #0
         BEQ     Set48MHzClock_waitloop1

         ; Switch main clock to PLL clock
         ;    0 - IRC
         ;    1 - Input clock to system PLL
         ;    2 - WDT clock
         ;    3 - System PLL output
         ; LPC_SYSCON->MAINCLKSEL = 3;
         MOVS    R1, #0x3
         STR     R1,[R0, #MAINCLKSEL]

         ; Update Main Clock Select setting (0->1 sequence)
         ; LPC_SYSCON->MAINCLKUEN = 0;
         MOVS    R1, #0x0
         STR     R1,[R0, #MAINCLKUEN]

         ; LPC_SYSCON->MAINCLKUEN = 1;
         MOVS    R1, #0x1
         STR     R1,[R0, #MAINCLKUEN]
         BX      LR
         ENDFUNC
; --------------------------------------------------
         ALIGN
         END
; --------------------------------------------------
