#include "LPC11XX.h"
#define KEIL_MCB1000_BOARD

// Function declarations
void LedOutputCfg(void);       // Set I/O pin connected to LED as output
void Timer0_Intr_Config(void); // Setup timer

int main(void)
{
  SystemInit();  // Switch Clock to 12MHz 
                 // (use internal RC oscillator only, PLL disabled)
  // Initialize LED output
  LedOutputCfg();  
  Timer0_Intr_Config();     // Program timer interrupt 2 Hz
  SCB->SCR = SCB->SCR | 0x2;// Turn on Sleep-On-Exit feature
  while(1) {
    __WFI(); // Enter sleep mode
    };
} // end main

void Timer0_Intr_Config(void)
{ // Use 32-bit timer 0
  // Enable clock to 32-bit timer 0
  LPC_SYSCON->SYSAHBCLKCTRL = LPC_SYSCON->SYSAHBCLKCTRL | (1<<9);

  LPC_TMR32B0->TCR = 0;        // Disable timer
  LPC_TMR32B0->PR  = 0;        // Prescaler set to 0 (TC increment every cycle)
  LPC_TMR32B0->PC  = 0;        // Prescaler counter current value clear
  LPC_TMR32B0->TC  = 0;        // Timer counter current value clear
  LPC_TMR32B0->MR0 = 5999999;  // Match Register set to "6 million - 1"
  LPC_TMR32B0->MCR = 3;        // When match MR0, generate interrupt and reset
  LPC_TMR32B0->TCR = 1;        // Enable timer
  NVIC_EnableIRQ(TIMER_32_0_IRQn); // Enable 32-bit timer 0 interrupt
  return;
}

void TIMER32_0_IRQHandler(void)
{
  LPC_TMR32B0->IR             = LPC_TMR32B0->IR;  // Clear interrupt
#ifdef KEIL_MCB1000_BOARD
  // For Keil MCB1000, use P2.0 for LED output
  LPC_GPIO2->MASKED_ACCESS[1] = ~LPC_GPIO2->MASKED_ACCESS[1]; // Toggle bit 0
#else
  // For LPCXpresso, use P0.7 for LED output
  LPC_GPIO0->MASKED_ACCESS[1<<7] = ~LPC_GPIO0->MASKED_ACCESS[1<<7]; // Toggle bit 7 
#endif
  return;
}

// Switch LED signal (P2_0 ) to output port with no pull up or pulldown
void LedOutputCfg(void)
{
  // Enable clock to IO configuration block (bit[16] of AHBCLOCK Control register)
  // and enable clock to GPIO (bit[6] of AHBCLOCK Control register
  LPC_SYSCON->SYSAHBCLKCTRL = LPC_SYSCON->SYSAHBCLKCTRL | (1<<16) | (1<<6);

#ifdef KEIL_MCB1000_BOARD
  // For Keil MCB1000, use P2.0 for LED output
  // PIO2_0 IO output config
  //  bit[5]   - Hysteresis (0=disable, 1 =enable)
  //  bit[4:3] - MODE(0=inactive, 1 =pulldown, 2=pullup, 3=repeater)
  //  bit[2:0] - Function (0 = IO, 1=DTR, 2=SSEL1)
  LPC_IOCON->PIO2_0 = (0<<5) + (0<<3) + (0x0);

  // Initial bit 0 output is 0
  LPC_GPIO2->MASKED_ACCESS[1] = 0;
  // Set pin 7 to 0 as output
  LPC_GPIO2->DIR = LPC_GPIO2->DIR | 0x1;
#else
  // For LPCXpresso, use P0.7 for LED output
  // PIO0_7 IO output config
  //  bit[5]   - Hysteresis (0=disable, 1 =enable)
  //  bit[4:3] - MODE(0=inactive, 1 =pulldown, 2=pullup, 3=repeater)
  //  bit[2:0] - Function (0 = IO, 1=CTS)
  LPC_IOCON->PIO0_7 = (0x0) + (0<<3) + (0<<5);
  // Initial bit[7] output is 0
  LPC_GPIO0->MASKED_ACCESS[1<<7] = 0;
  // Set pin 7 as output
  LPC_GPIO0->DIR = LPC_GPIO0->DIR | (1<<7);
#endif
  return;
} // end LedOutputCfg
