#include "LPC11XX.h"
#include "uart_io.h"
#include <stdio.h>

// Define SVC function
int __svc(0x00)  svc_service_add(int x, int y);
int __svc(0x01)  svc_service_sub(int x, int y);
int __svc(0x02)  svc_service_incr(int x);

void SVC_Handler_main(unsigned int * svc_args);

// Function declarations
int main(void)
{
  int x, y, z;
 
  SystemInit();  // System Initialization
  UartConfig();  // Initialize UART

  x = 3; y = 5;
  z = svc_service_add(x, y);
  printf ("3+5 = %d \n", z);

  x = 9; y = 2;
  z = svc_service_sub(x, y);
  printf ("9-2 = %d \n", z);

  x = 3; 
  z = svc_service_incr(x);
  printf ("3++ = %d \n", z);

  while(1);
}

// SVC handler - Assembly wrapper to extract 
//               stack frame starting address
__asm void SVC_Handler(void)
{
  MOVS   r0, #4
  MOV    r1, LR
  TST    r0, r1
  BEQ    stacking_used_MSP
  MRS    R0, PSP ; first parameter - stacking was using PSP
  LDR    R1,=__cpp(SVC_Handler_main)
  BX     R1
stacking_used_MSP
  MRS    R0, MSP ; first parameter - stacking was using MSP
  LDR    R1,=__cpp(SVC_Handler_main)
  BX     R1
}

// SVC handler - main code to handle processing
// Input parameter is stack frame starting address
// obtained from assembly wrapper.
void SVC_Handler_main(unsigned int * svc_args)
{
  // Stack frame contains:
  // r0, r1, r2, r3, r12, r14, the return address and xPSR
  // - Stacked R0  = svc_args[0]
  // - Stacked R1  = svc_args[1]
  // - Stacked R2  = svc_args[2]
  // - Stacked R3  = svc_args[3]
  // - Stacked R12 = svc_args[4]
  // - Stacked LR  = svc_args[5]
  // - Stacked PC  = svc_args[6]
  // - Stacked xPSR= svc_args[7]

  unsigned int svc_number;
  svc_number = ((char *)svc_args[6])[-2];
  switch(svc_number)
    {
    case 0: svc_args[0] = svc_args[0] + svc_args[1];
            break;
    case 1: svc_args[0] = svc_args[0] - svc_args[1];
            break;
    case 2: svc_args[0] = svc_args[0] + 1;
            break;
    default: // Unknown SVC request
            break;
    }
  return;
}
