#include "LPC11XX.h"
#include "uart_io.h"
#include <stdio.h>
void Timer0_Intr_Config(void); // declare timer initialization function

int main(void)
{
  SystemInit();  // System Initialization
  UartConfig();  // Initialize UART

  NVIC_SetPriority(TIMER_32_0_IRQn, 0x0); // Set Timer  to highest level
  NVIC_SetPriority(PendSV_IRQn , 0x3);    // Set PendSV to lowest  level

  // Program timer interrupt at 1 Hz.
  // At 48MHz, Timer trigger every 4800000 CPU cycles
  Timer0_Intr_Config();

  while(1);
}

void PendSV_Handler(void)
{
  printf ("[PendSV] Timer interrupt triggered\n");
  return;
}

void TIMER32_0_IRQHandler(void)
{
  LPC_TMR32B0->IR = LPC_TMR32B0->IR;  // Clear interrupt
  SCB->ICSR       = SCB->ICSR | 0x10000000; // Set PendSV pending status
  return;
}

void Timer0_Intr_Config(void)
{ // Use 32-bit timer 0
  // Enable clock to 32-bit timer 0
  LPC_SYSCON->SYSAHBCLKCTRL = LPC_SYSCON->SYSAHBCLKCTRL | (1<<9);

  LPC_TMR32B0->TCR = 0;        // Disable timer
  LPC_TMR32B0->PR  = 0;        // Prescaler set to 0 (TC increment every cycle)
  LPC_TMR32B0->PC  = 0;        // Prescaler counter current value clear
  LPC_TMR32B0->TC  = 0;        // Timer counter current value clear
  LPC_TMR32B0->MR0 = 47999999; // Match Register set to "48 million - 1"
  LPC_TMR32B0->MCR = 3;        // When match MR0, generate interrupt and reset
  LPC_TMR32B0->TCR = 1;        // Enable timer
  NVIC_EnableIRQ(TIMER_32_0_IRQn); // Enable 32-bit timer 0 interrupt
  return;
}
