#include <RTL.h>
#include "LPC11XX.h"
#include "uart_io.h"
#include <stdio.h>

OS_TID t_task1; // Declare a task ID for task1
OS_TID t_task2; // Declare a task ID for task2
OS_MUT UartLock; // Declare a MUTEX for Uart access

__task void task1(void) {
  while(1) {   
    os_dly_wait(50);
    os_mut_wait(UartLock, 0xFFFF); // Wait for UART access lock,
                                   // with indefinite timeout period
    printf ("Task 1 running.\n");
    os_mut_release(UartLock);      // Finished using UART, MUTEX release
    }
  }

__task void task2(void) {
  while (1) {
    os_dly_wait(50);
    os_mut_wait(UartLock, 0xFFFF); // Wait for UART access lock,
                                   // with indefinite timeout period
    printf ("Task 2 running.\n");
    os_mut_release(UartLock);      // Finished using UART, MUTEX release
    }
  }

__task void init (void) {
  os_mut_init(UartLock);   // Create a Mutex called UartLock
  t_task1 = os_tsk_create (task1, 1); // Create a task "task1" with priority 1
  t_task2 = os_tsk_create (task2, 1); // Create a task "task2" with priority 1
  os_tsk_delete_self ();
}  

int main(void)
{
  SystemInit();    // Switch Clock to 48MHz 
  UartConfig();    // Configure UART
  os_sys_init(init); // Initialize OS
} // end main
