#include <RTL.h>
#include "LPC11XX.h"
#include "uart_io.h"
#include <stdio.h>

OS_TID t_sendmsg_1; // Declare a task ID for task1
OS_TID t_sendmsg_2; // Declare a task ID for task2
OS_TID t_printmsg;  // Declare a task ID for printmsg

// define a memory pool for message box
os_mbx_declare (mailbox1, 16);

// Declare 16 blocks of 20 bytes for message contents
// each message contains two integers
_declare_box(mpool, 20, 16);

__task void send_msg_1(void) { // sender of message
  unsigned int *msg;
  int          counter=0;
  while(1) {
    os_dly_wait(100);
    msg = _alloc_box(mpool); // allocate block for message content
    if (*msg == 0) {
     printf ("_alloc_box failed\n");
     while (1);}
    msg[0] = counter;// Message
    msg[1] = 0x1234; // Message
    os_mbx_send(mailbox1, (void *) msg, 0xFFFF); // Send to mail box
    counter ++;
    }
  }

__task void send_msg_2(void) { // sender of message
  unsigned int *msg;
  int          counter=0x100;
  while(1) {
    os_dly_wait(100);
    msg = _alloc_box(mpool); // allocate block for message content
    if (*msg == 0) {
     printf ("_alloc_box failed\n");
     while (1);}
    msg[0] = counter;// Message
    msg[1] = 0x4567; // Message
    os_mbx_send(mailbox1, (void *) msg, 0xFFFF); // Send to mail box
    counter ++;
    }
  }

__task void print_msg(void) { // receiver of message
  unsigned int *msg;
  unsigned int received_values[2];
  while (1) {
    os_mbx_wait(mailbox1, (void *)&msg, 0xFFFF);
    received_values[0] = msg[0];
    received_values[1] = msg[1];
    _free_box(mpool, msg);
    printf ("Received values %x, %x\n",received_values[0],received_values[1]);
    }
  }

__task void init (void) {
  int          status;

  // Initial fixed block size memory pool,  // block size is 20
  status = _init_box (mpool, sizeof (mpool), 20);
  if (status != 0) {
   printf ("_init_box failed\n");
   while (1);}

  os_mbx_init(mailbox1, sizeof(mailbox1)); // Initialize mailbox1  
  // Create a task "send_msg_1" with priority 1
  t_sendmsg_1= os_tsk_create (send_msg_1,1);
  // Create a task "send_msg_2" with priority 1
  t_sendmsg_2= os_tsk_create (send_msg_2,1);
  // Create a task "print_msg"  with priority 1
  t_printmsg = os_tsk_create (print_msg, 1);
  os_tsk_delete_self ();
}  

int main(void)
{
  SystemInit();    // Switch Clock to 48MHz 
  UartConfig();    // Configure UART
  os_sys_init(init); // Initialize OS
} // end main

