#include "LPC11XX.h"
#define KEIL_MCB1000_BOARD

// Function declarations
void LedOutputCfg(void);  // Set I/O pin connected to LED as output

int main(void)
{
  // Switch Clock to 48MHz
  SystemInit();
#ifdef KEIL_MCB1000_BOARD
  // Initialize LED (GPIO #2, bit[0]) output
#else
  // Initialize LED (GPIO #0, bit[7]) output
#endif
  LedOutputCfg();
  // Program SysTick timer interrupt at 1KHz.
  // At 48MHz, SysTick trigger every 48000 CPU cycles
  SysTick_Config(48000);

  while(1);
} // end main

// SysTick handler to toggle LED every 500 ticks
void SysTick_Handler(void)
{
static short int TickCount = 0;
if ((TickCount++) == 500) { // for every 500 counts, toggle LED
  TickCount = 0; // reset counter to 0
#ifdef KEIL_MCB1000_BOARD
  // For Keil MCB1000, use P2.0 for LED output
  LPC_GPIO2->MASKED_ACCESS[1] = ~LPC_GPIO2->MASKED_ACCESS[1]; // Toggle bit 0
#else
  // For LPCXpresso, use P0.7 for LED output
  LPC_GPIO0->MASKED_ACCESS[1<<7] = ~LPC_GPIO0->MASKED_ACCESS[1<<7]; // Toggle bit 7 
#endif
  }
return;
}
// Switch LED signal (P0_7) to output port with no pull up or pulldown
void LedOutputCfg(void)
{
  // Enable clock to IO configuration block (bit[16] of AHBCLOCK Control register)
  // and enable clock to GPIO (bit[6] of AHBCLOCK Control register
  LPC_SYSCON->SYSAHBCLKCTRL = LPC_SYSCON->SYSAHBCLKCTRL | (1<<16) | (1<<6);


#ifdef KEIL_MCB1000_BOARD
  // For Keil MCB1000, use P2.0 for LED output
  // PIO2_0 IO output config
  //  bit[5]   - Hysteresis (0=disable, 1 =enable)
  //  bit[4:3] - MODE(0=inactive, 1 =pulldown, 2=pullup, 3=repeater)
  //  bit[2:0] - Function (0 = IO, 1=DTR, 2=SSEL1)
  LPC_IOCON->PIO2_0 = (0<<5) + (0<<3) + (0x0);

  // Initial bit 0 output is 0
  LPC_GPIO2->MASKED_ACCESS[1] = 0;
  // Set pin 7 to 0 as output
  LPC_GPIO2->DIR = LPC_GPIO2->DIR | 0x1;
#else
  // For LPCXpresso, use P0.7 for LED output
  // PIO0_7 IO output config
  //  bit[5]   - Hysteresis (0=disable, 1 =enable)
  //  bit[4:3] - MODE(0=inactive, 1 =pulldown, 2=pullup, 3=repeater)
  //  bit[2:0] - Function (0 = IO, 1=CTS)
  LPC_IOCON->PIO0_7 = (0x0) + (0<<3) + (0<<5);
  // Initial bit[7] output is 0
  LPC_GPIO0->MASKED_ACCESS[1<<7] = 0;
  // Set pin 7 as output
  LPC_GPIO0->DIR = LPC_GPIO0->DIR | (1<<7);
#endif
  return;
} // end LedOutputCfg
