; Base Address Constant and register offset definition
; Cortex-M0
SCS_BASE              EQU    0xE000E000
SysTick_BASE          EQU    0xE000E010
NVIC_BASE             EQU    0xE000E100
SCB_BASE              EQU    0xE000ED00

; Core peripheral address offset
; 1) SysTick (use SysTick_BASE)
SysTick_CTRL          EQU    0x000
SysTick_LOAD          EQU    0x004
SysTick_VAL           EQU    0x008
SysTick_CALIB         EQU    0x00C

; 2) NVIC (use NVIC_BASE)
NVIC_ISER             EQU    0x000  ; 0xE000E100
NVIC_ICER             EQU    0x080  ; 0xE000E180
NVIC_ISPR             EQU    0x100  ; 0xE000E200
NVIC_ICPR             EQU    0x180  ; 0xE000E280
NVIC_IPR0             EQU    0x300  ; 0xE000E400

; 3) SCB (use SCB_BASE)
SCB_CPUID             EQU    0x000
SCB_ICSR              EQU    0x004
SCB_AIRCR             EQU    0x00C
SCB_SCR               EQU    0x010
SCB_CCR               EQU    0x014
SCB_SHP0              EQU    0x01C
SCB_SHP1              EQU    0x020
SCB_SHCSR             EQU    0x024
SCB_DFSR              EQU    0x030

; LPC111x specific peripheral base address
; APB0 peripherals    
LPC_I2C_BASE          EQU    0x40000000
LPC_WDT_BASE          EQU    0x40004000
LPC_UART_BASE         EQU    0x40008000
LPC_CT16B0_BASE       EQU    0x4000C000
LPC_CT16B1_BASE       EQU    0x40010000
LPC_CT32B0_BASE       EQU    0x40014000
LPC_CT32B1_BASE       EQU    0x40018000
LPC_ADC_BASE          EQU    0x4001C000
LPC_PMU_BASE          EQU    0x40038000
LPC_SSP0_BASE         EQU    0x40040000
LPC_IOCON_BASE        EQU    0x40044000
LPC_SYSCON_BASE       EQU    0x40048000
LPC_SSP1_BASE         EQU    0x40058000

; AHB peripherals
LPC_GPIO_BASE         EQU    0x50000000
LPC_GPIO0_BASE        EQU    0x50000000
LPC_GPIO1_BASE        EQU    0x50010000
LPC_GPIO2_BASE        EQU    0x50020000
LPC_GPIO3_BASE        EQU    0x50030000

; Register offset
; 1) SYSCON
SYSMEMREMAP           EQU    0x000
PRESETCTRL            EQU    0x004
SYSPLLCTRL            EQU    0x008
SYSPLLSTAT            EQU    0x00C
SYSOSCCTRL            EQU    0x020
WDTOSCCTRL            EQU    0x024
IRCCTRL               EQU    0x028
SYSRESSTAT            EQU    0x030
SYSPLLCLKSEL          EQU    0x040
SYSPLLCLKUEN          EQU    0x044
MAINCLKSEL            EQU    0x070
MAINCLKUEN            EQU    0x074
SYSAHBCLKDIV          EQU    0x078
SYSAHBCLKCTRL         EQU    0x080
SSP0CLKDIV            EQU    0x094
UARTCLKDIV            EQU    0x098
SSP1CLKDIV            EQU    0x09C
SYSTICKCLKDIV         EQU    0x0B0
WDTCLKSEL             EQU    0x0D0
WDTCLKUEN             EQU    0x0D4
WDTCLKDIV             EQU    0x0D8
CLKOUTCLKSEL          EQU    0x0E0
CLKOUTUEN             EQU    0x0E4
CLKOUTDIV             EQU    0x0E8
PIOPORCAP0            EQU    0x100
PIOPORCAP1            EQU    0x104
BODCTRL               EQU    0x150
SYSTCKCAL             EQU    0x158
STARTAPRP0            EQU    0x200
STARTERP0             EQU    0x204
STARTRSRP0CLR         EQU    0x208
STARTSRP0             EQU    0x20C

PDSLEEPCFG            EQU    0x230
PDAWAKECFG            EQU    0x234
PDRUNCFG              EQU    0x238

DEVICE_ID             EQU    0x3F4

; 2) IOCON register offset
PIO2_6                EQU    0x000
PIO2_0                EQU    0x008
PIO0_1                EQU    0x010
PIO1_8                EQU    0x014
PIO0_2                EQU    0x01C

PIO2_7                EQU    0x020
PIO2_8                EQU    0x024
PIO2_1                EQU    0x028
PIO0_3                EQU    0x02C
PIO0_4                EQU    0x030
PIO0_5                EQU    0x034
PIO1_9                EQU    0x038
PIO3_4                EQU    0x03C

PIO2_4                EQU    0x040
PIO2_5                EQU    0x044
PIO3_5                EQU    0x048
PIO0_6                EQU    0x04C
PIO0_7                EQU    0x050
PIO2_9                EQU    0x054
PIO2_10               EQU    0x058
PIO2_2                EQU    0x05C

PIO0_8                EQU    0x060
PIO0_9                EQU    0x064
JTAG_TCK_PIO0_10      EQU    0x068
PIO1_10               EQU    0x06C
PIO2_11               EQU    0x070
JTAG_TDI_PIO0_11      EQU    0x074
JTAG_TMS_PIO1_0       EQU    0x078
JTAG_TDO_PIO1_1       EQU    0x07C
JTAG_nTRST_PIO1_2     EQU    0x080
PIO3_0                EQU    0x084
PIO3_1                EQU    0x08C
PIO2_3                EQU    0x08C
ARM_SWDIO_PIO1_3      EQU    0x090
PIO1_4                EQU    0x094
PIO1_11               EQU    0x098
PIO3_2                EQU    0x09C

PIO1_5                EQU    0x0A0
PIO1_6                EQU    0x0A4
PIO1_7                EQU    0x0A8
PIO3_3                EQU    0x0AC
SCK_LOC               EQU    0x0B0
DSR_LOC               EQU    0x0B4
DCD_LOC               EQU    0x0B8
RI_LOC                EQU    0x0BC

; 3) PMU
PMU_PCON              EQU    0x000
PMU_GPREG0            EQU    0x004
PMU_GPREG1            EQU    0x008
PMU_GPREG2            EQU    0x00C
PMU_GPREG3            EQU    0x010
PMU_GPREG4            EQU    0x014

; 4) GPIO
; Most control registers are start from offset 0x8000
LPC_GPIO0_REGBASE     EQU    LPC_GPIO0_BASE + 0x8000
LPC_GPIO1_REGBASE     EQU    LPC_GPIO1_BASE + 0x8000
LPC_GPIO2_REGBASE     EQU    LPC_GPIO2_BASE + 0x8000
LPC_GPIO3_REGBASE     EQU    LPC_GPIO3_BASE + 0x8000

; Since data register offset is large, use fixed constant
; instead.
LPC_GPIO0_DATA        EQU    0x50000000 + 0x3FFC
LPC_GPIO1_DATA        EQU    0x50010000 + 0x3FFC
LPC_GPIO2_DATA        EQU    0x50020000 + 0x3FFC
LPC_GPIO3_DATA        EQU    0x50030000 + 0x3FFC

; Masked access use LPC_GPIO0/1/2/3_BASE
GPIO_MASKED_ACCESS_BASE    EQU    0x000

; Following are control register offset
; (Use LPC_GPIO0/1/2/3_REGBASE)
GPIO_DIR              EQU    0x000 ; 0x8000
GPIO_IS               EQU    0x004 ; 0x8004
GPIO_IBE              EQU    0x008 ; 0x8008
GPIO_IEV              EQU    0x00C ; 0x800C
GPIO_IE               EQU    0x010 ; 0x8010
GPIO_RIS              EQU    0x014 ; 0x8014
GPIO_MIS              EQU    0x018 ; 0x8018
GPIO_IC               EQU    0x01C ; 0x801C

; 5) Timer register offset
TIMER_IR              EQU    0x000
TIMER_TCR             EQU    0x004
TIMER_TC              EQU    0x008
TIMER_PR              EQU    0x00C
TIMER_PC              EQU    0x010
TIMER_MCR             EQU    0x014
TIMER_MR0             EQU    0x018
TIMER_MR1             EQU    0x01C
TIMER_MR2             EQU    0x020
TIMER_MR3             EQU    0x024
TIMER_CCR             EQU    0x028
TIMER_CR0             EQU    0x02C
TIMER_EMR             EQU    0x03C
TIMER_CTCR            EQU    0x070
TIMER_PWMC            EQU    0x074

; 6) UART register offset
UART_RBR              EQU    0x000
UART_THR              EQU    0x000
UART_DLL              EQU    0x000
UART_DLM              EQU    0x004
UART_IER              EQU    0x004
UART_IIR              EQU    0x008
UART_FCR              EQU    0x008
UART_LCR              EQU    0x00C
UART_MCR              EQU    0x010
UART_LSR              EQU    0x014
UART_MSR              EQU    0x018
UART_SCR              EQU    0x01C
UART_ACR              EQU    0x020
UART_FDR              EQU    0x028
UART_TER              EQU    0x030
UART_RS485CTRL        EQU    0x04C
UART_ADRMATCH         EQU    0x050
UART_RS485DLY         EQU    0x054
UART_FIFOLVL          EQU    0x058

; 7) SSP register offset
SSP_CR0               EQU    0x000
SSP_CR1               EQU    0x004
SSP_DR                EQU    0x008
SSP_SR                EQU    0x00C
SSP_CPSR              EQU    0x010
SSP_IMSC              EQU    0x014
SSP_RIS               EQU    0x018
SSP_MIS               EQU    0x01C
SSP_ICR               EQU    0x020

; 8) I2C register offset
I2C_CONSET            EQU    0x000
I2C_STAT              EQU    0x004
I2C_DAT               EQU    0x008
I2C_ADR0              EQU    0x00C
I2C_SCLH              EQU    0x010
I2C_SCLL              EQU    0x014
I2C_CONCLR            EQU    0x018
I2C_MMCTRL            EQU    0x01C
I2C_ADR1              EQU    0x020
I2C_ADR2              EQU    0x024
I2C_ADR3              EQU    0x028
I2C_DATA_BUFFER       EQU    0x02C
I2C_MASK0             EQU    0x030
I2C_MASK1             EQU    0x034
I2C_MASK2             EQU    0x038
I2C_MASK3             EQU    0x03C

; 9) Watchdog
WDT_MOD               EQU    0x000
WDT_TC                EQU    0x004
WDT_FEED              EQU    0x008
WDT_TV                EQU    0x00C

; 10) ADC register offset
ADC_CR                EQU    0x000
ADC_GDR               EQU    0x004
ADC_INTEN             EQU    0x00C
ADC_DR0               EQU    0x010
ADC_DR1               EQU    0x014
ADC_DR2               EQU    0x018
ADC_DR3               EQU    0x01C
ADC_DR4               EQU    0x020
ADC_DR5               EQU    0x024
ADC_DR6               EQU    0x028
ADC_DR7               EQU    0x02C
ADC_STAT              EQU    0x030
                      END