#include "LPC11XX.h"
#include <stdio.h>
#include "uart_io.h"

#define INVALID_ADDRESS (*((volatile unsigned long *)(0xFFFF0000)))

int main(void)
{
  SystemInit();  // Switch Clock to 48MHz
  UartConfig();
  UartPuts("Hardfault handler demo\n");

  /* Generate fault */
  INVALID_ADDRESS = INVALID_ADDRESS + 1;

  while(1);
} // end main

void HardFault_Handler_c(unsigned int * hf_args)
{
  // Stack frame contains:
  // r0, r1, r2, r3, r12, r14, the return address and xPSR
  // - Stacked R0  = hf_args[0]
  // - Stacked R1  = hf_args[1]
  // - Stacked R2  = hf_args[2]
  // - Stacked R3  = hf_args[3]
  // - Stacked R12 = hf_args[4]
  // - Stacked LR  = hf_args[5]
  // - Stacked PC  = hf_args[6]
  // - Stacked xPSR= hf_args[7]

  UartPuts("HardFault Handler:\n");

  UartPuts("-  R0 = 0x");
  UartPutHex(hf_args[0]);
  UartPutc('\n');

  UartPuts("-  R1 = 0x");
  UartPutHex(hf_args[1]);
  UartPutc('\n');

  UartPuts("-  R2 = 0x");
  UartPutHex(hf_args[2]);
  UartPutc('\n');

  UartPuts("-  R3 = 0x");
  UartPutHex(hf_args[3]);
  UartPutc('\n');

  UartPuts("- R12 = 0x");
  UartPutHex(hf_args[4]);
  UartPutc('\n');

  UartPuts("-  LR = 0x");
  UartPutHex(hf_args[5]);
  UartPutc('\n');

  UartPuts("-  PC = 0x");
  UartPutHex(hf_args[6]);
  UartPutc('\n');

  UartPuts("- xPSR= 0x");
  UartPutHex(hf_args[7]);
  UartPutc('\n');

  while (1);
  return;
}

