#include "LPC11XX.h"
#define KEIL_MCB1000_BOARD

// Function declarations
void LedOutputCfg(void);  // Set I/O pin connected to LED as output

int main(void)
{
  // Clock remain at 12MHz
  SystemInit();
  // Initialize LED output
  LedOutputCfg();
  // Program SysTick timer to generate an interrupt after 0xFFFFFF cycles.
  // At 12MHz, SysTick trigger after 1.4 second
  SysTick->LOAD = 0xFFFFFF;
  SysTick->VAL  = 0x0;
  SysTick->CTRL = 0x7; // Enable SysTick with exception generation
                       // and use core clock as source
  while(1);
} // end main

// SysTick handler to toggle LED and disable SysTick
void SysTick_Handler(void)
{  
#ifdef KEIL_MCB1000_BOARD
  // For Keil MCB1000, use P2.0 for LED output
  LPC_GPIO2->MASKED_ACCESS[1] = ~LPC_GPIO2->MASKED_ACCESS[1]; // Toggle bit 0
#else
  LPC_GPIO0->MASKED_ACCESS[1<<7] = ~LPC_GPIO0->MASKED_ACCESS[1<<7]; // Toggle bit 7 
#endif
  
  // Disable SysTick
  SysTick->CTRL = 0;
  // Clear SysTick pending status in case it has already been triggered
  SCB->ICSR = SCB->ICSR | (1<<25); // Set PENDSTCLR
  return;
}
// Switch LED signal (P0_7) to output port with no pull up or pulldown
void LedOutputCfg(void)
{
  // Enable clock to IO configuration block (bit[16] of AHBCLOCK Control register)
  // and enable clock to GPIO (bit[6] of AHBCLOCK Control register
  LPC_SYSCON->SYSAHBCLKCTRL = LPC_SYSCON->SYSAHBCLKCTRL | (1<<16) | (1<<6);

#ifdef KEIL_MCB1000_BOARD
  // For Keil MCB1000, use P2.0 for LED output
  // PIO2_0 IO output config
  //  bit[5]   - Hysteresis (0=disable, 1 =enable)
  //  bit[4:3] - MODE(0=inactive, 1 =pulldown, 2=pullup, 3=repeater)
  //  bit[2:0] - Function (0 = IO, 1=DTR, 2=SSEL1)
  LPC_IOCON->PIO2_0 = (0<<5) + (0<<3) + (0x0);

  // Initial bit 0 output is 0
  LPC_GPIO2->MASKED_ACCESS[1] = 0;
  // Set pin 7 to 0 as output
  LPC_GPIO2->DIR = LPC_GPIO2->DIR | 0x1;
#else
  // For LPCXpresso, use P0.7 for LED output
  // PIO0_7 IO output config
  //  bit[5]   - Hysteresis (0=disable, 1 =enable)
  //  bit[4:3] - MODE(0=inactive, 1 =pulldown, 2=pullup, 3=repeater)
  //  bit[2:0] - Function (0 = IO, 1=CTS)
  LPC_IOCON->PIO0_7 = (0x0) + (0<<3) + (0<<5);
  // Initial bit[7] output is 0
  LPC_GPIO0->MASKED_ACCESS[1<<7] = 0;
  // Set pin 7 as output
  LPC_GPIO0->DIR = LPC_GPIO0->DIR | (1<<7);
#endif
  return;
} // end LedOutputCfg
