#include "LPC11XX.h"

// Function declarations
void          UartConfig(void);         // Uart configuration
unsigned char UartPutc(unsigned char my_ch);     // Uart character output
void          UartPuts(unsigned char * mytext);  // Uart string output
int           UartGetRxDataAvail(void); // Detect if new data are received
unsigned char UartGetRxData(void);      // Get received data from Uart
unsigned char UartGetc(void);           // Uart character input
void          UartPutHex(unsigned int din); // Hexadecimal display
void          UartPutDec(unsigned int din); // Decimal display
int UartGets(char dest[], int length);

// Start of main program
int main(void)
{
  char textbuf[20];
  int  return_state;

  // System Initialization
  SystemInit();
  // Initialize UART
  UartConfig();

  UartPutc('\n');
  UartPuts ("Testing UartPuts    : Hello world\n");
  UartPuts ("Testing UartPutHex  : 0x12345678 = 0x");
  UartPutHex(0x12345678);
  UartPutc('\n');
  UartPuts ("Testing UartPutHex  : 0x0        = 0x");
  UartPutHex(0x0);
  UartPutc('\n');
  UartPuts ("Testing UartPutHex  : 0xFFFFFFFF = 0x");
  UartPutHex(0xFFFFFFFF);
  UartPutc('\n');
 
  UartPutc('\n');
  UartPuts ("Testing UartPutDec  : 987654321  = ");
  UartPutDec(987654321);
  UartPutc('\n');

  UartPuts ("Testing UartPutDec  : 0          = ");
  UartPutDec(0);
  UartPutc('\n');

  UartPuts ("Testing UartPutDec  : 11         = ");
  UartPutDec(11);
  UartPutc('\n');

  UartPuts ("Testing UartPutDec  : 100        = ");
  UartPutDec(100);
  UartPutc('\n');

  UartPuts ("Testing UartPutDec  : 1001       = ");
  UartPutDec(1001);
  UartPutc('\n');

  UartPuts ("Testing UartPutDec  : 12345      = ");
  UartPutDec(12345);
  UartPutc('\n');

  UartPuts ("Testing UartPutDec  : 100000000  = ");
  UartPutDec(100000000);
  UartPutc('\n');

  UartPuts ("Testing UartPutDec  : 4294967295  = ");
  UartPutDec(4294967295U);
  UartPutc('\n');

  while (1) {
  UartPutc('\n');
  UartPuts ("String input test : ");
  return_state = UartGets(&textbuf[0], 19);
  if (return_state!=0) {
    UartPuts ("\nESC pressed :");
    }
  else {
    UartPuts ("\nInput was   :");
    }
  UartPuts ((unsigned char *)textbuf);
  UartPutc('\n');
  };
 
 

} // end main


void UartConfig(void)
{
  // UART interface are : PIO1_7 (TXD) and PIC1_6 (RXD)
  // Other UART signals (DTR, DSR, CTS, RTS, RI) are not used

  // Enable clock to IO configuration block (bit[16] of AHBCLOCK Control register
  LPC_SYSCON->SYSAHBCLKCTRL = LPC_SYSCON->SYSAHBCLKCTRL | (1<<16);

  // PIO1_7 IO output config
  //  bit[5]   - Hysteresis (0=disable, 1 =enable)
  //  bit[4:3] - MODE(0=inactive, 1 =pulldown, 2=pullup, 3=repeater)
  //  bit[2:0] - Function (0 = IO, 1=TXD, 2=CT32B0_MAT1)
  LPC_IOCON->PIO1_7 = (0x1) + (0<<3) + (0<<5);
  // PIO1_6 IO input config
  //  bit[5]   - Hysteresis (0=disable, 1 =enable)
  //  bit[4:3] - MODE(0=inactive, 1 =pulldown, 2=pullup, 3=repeater)
  //  bit[2:0] - Function (0 = IO, 1=RXD, 2=CT32B0_MAT0)
  LPC_IOCON->PIO1_6 = (0x1) + (2<<3) + (1<<5);

  // Enable clock to UART (bit[12] of AHBCLOCK Control register
  LPC_SYSCON->SYSAHBCLKCTRL = LPC_SYSCON->SYSAHBCLKCTRL | (1<<12);
  // UART_PCLK divide ratio = 1
  LPC_SYSCON->UARTCLKDIV = 1;

  // UART_PCLK = 48MHz, Baudrate = 38400, divide ratio = 1250
  // Line Control Register
  LPC_UART->LCR = (1<<7) |   // Enable access to Divisor Latches
                  (0<<6) |   // Disable Break COntrol
                  (0<<4) |   // Bit[5:4] parity select (odd, evem, sticky-1, sticky-0)
                  (0<<3) |   // parity disabled
                  (0<<2) |   // 1 stop bit
                  (3<<0);    // 8-bit data
  LPC_UART->DLL = 78;  // Divsor Latch Least Significant Byte : 48MHz/38400/16 = 78.125
  LPC_UART->DLM = 0;   // Divsor Latch Most Significant Byte  : 0
  LPC_UART->LCR = (0<<7) |   // Disable access to Divisor Latches
                  (0<<6) |   // Disable Break COntrol
                  (0<<4) |   // Bit[5:4] parity select (odd, evem, sticky-1, sticky-0)
                  (0<<3) |   // parity disabled
                  (0<<2) |   // 1 stop bit
                  (3<<0);    // 8-bit data

  LPC_UART->FCR = 1; // Enable FIFO

  return;
}
// Get received data
__inline unsigned char UartGetRxData(void)
{
  return ((char)LPC_UART->RBR);
}
// Detect if new received data are available
__inline int  UartGetRxDataAvail(void)
{
  return (LPC_UART->LSR & 0x1);
}
// Output a character
unsigned char UartPutc(unsigned char my_ch)
{
  if (my_ch == '\n') {
    while ((LPC_UART->LSR & (1<<5))==0); // Wait if Transmit Holding register is not empty
    LPC_UART->THR = 13;  // Output carriage return (for Windows Hyperterminal)
    }
  while ((LPC_UART->LSR & (1<<5))==0); // Wait if Transmit Holding register is not empty
  LPC_UART->THR = my_ch; // write to transmit holding register

  if (my_ch == 13) {
    while ((LPC_UART->LSR & (1<<5))==0); // Wait if Transmit Holding register is not empty
    LPC_UART->THR = 10;  // Output new line (for Windows Hyperterminal)
    }

  return (my_ch);
}
// Get a character from UART, if no data available then wait
unsigned char UartGetc(void)
{
  while (UartGetRxDataAvail()==0); // wait if receive buffer empty
  return ((char)LPC_UART->RBR);
}
// Uart string output
void UartPuts(unsigned char * mytext)
{
  unsigned char CurrChar;
  CurrChar = *mytext;
  while (CurrChar != (char) 0x0){
    UartPutc(CurrChar);  // Normal data
    mytext++;
    CurrChar = *mytext;
    }
  return;
}
void UartPutHex(unsigned int din)
{
unsigned int nmask = 0xF0000000U;
unsigned int nshift = 28;
unsigned short int data4bit;
  do {
   data4bit = (din & nmask) >> nshift;
   data4bit = data4bit+48; // convert data to ASCII
   if (data4bit>57)	data4bit = data4bit+7;
   UartPutc((char) data4bit);
   nshift = nshift - 4;
   nmask = nmask >> 4;
  } while (nmask!=0);
  return;

}

void UartPutDec(unsigned int din)
{
const unsigned int DecTable[10] = {
  1000000000,100000000,10000000,1000000, 
  100000, 10000, 1000, 100, 10, 1};

int count=0;// digital count
int n;      // calculation for each digital
// Remove preceding zeros
  while ((din < DecTable[count]) && (din>10)) {count++;}

  while (count<10) {
    n=0;
    while (din >= DecTable[count]) {
      din = din - DecTable[count];
      n++;
      }
    n = n + 48; // convert to ascii 0 to 9
    UartPutc((char) n);
    count++;
  };
  return;
}

int UartGets(char dest[], int length)
{
unsigned int textlen=0; // Current text length
char ch; // current character
do {
  ch = UartGetc(); // Get a character from UART
  switch (ch)	{
    case 8: // Back space
     if (textlen>0) {
       textlen--;
       UartPutc(ch); // Back space
       UartPutc(' '); // Replace last character with space on console
       UartPutc(ch); // Back space again to adjust cursor position
       }
      break;
    case 13: // Enter is pressed
     dest[textlen] = 0; // null terminate
     UartPutc(ch); // echo typed character
     break;
    case 27: // ESC is pressed
     dest[textlen] = 0; // null terminate
     UartPutc('\n');
     break;
    default: // if input length is within limit and input is valid
      if ((textlen<length) & 
        ((ch >= 0x20) & (ch < 0x7F))) // valid characters
         {
         dest[textlen] = ch; // append character to buffer
         textlen++;
         UartPutc(ch); // echo typed character
         }
      break;
     } // end switch
  } while ((ch!=13) && (ch!=27));
  if (ch==27) {
    return 1; // ESC key pressed
  } else {
    return 0; // Return key pressed
    }
}
